/**********************************************************
* FILE STACK.C version 1.00  - 05/10/94 -
*
* DESCRIPTION:
*  contains stack's functions.
*
* NOTES:
*
* Copyright 1993 by EZ Software 
*
* Permission to use, copy, modify, and distribute this software and its
* documentation for any purpose and without fee is hereby granted, provided 
* that the above copyright notice appear in all copies and that both that 
* copyright notice and this permission notice appear in supporting 
* documentation, and that the name of EZ Software 
* not be used in advertising or publicity pertaining to distribution of the 
* software without specific written prior permission. 
* EZ Software makes no representations about the 
* suitability of this software for any purpose.  It is provided "as is"
* without express or implied warranty.
*
* EZ Software DISCLAIMS ALL WARRANTIES WITH REGARD TO THIS 
* SOFTWARE, INCLUDING ALL IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS, 
* IN NO EVENT SHALL EZ Software BE LIABLE FOR ANY SPECIAL,
* INDIRECT OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES RESULTING FROM LOSS OF USE,
* DATA OR PROFITS, WHETHER IN AN ACTION CONTRACT, NEGLIGENCE OR OTHER TORTIOUS
* ACTION, ARISING OUT OF OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS 
* SOFTWARE.
*
*
*  Original Author: Laurent Chavey, EZ Software 
*           (chavey@cis.udel.edu)
*            june, 1993
*
**********************************************************/

#define STACK
#include "stack.h"

/********************* PRIMITIVES ***********************/

/*********************************************************
* FUNCTION: 
*  stknew
*
* PROTOTYPE:
*  stack *stknew(int );
*
* PARAMETERS:
*  int nDataSize   size in byte of the data element.
*
* RETURN:
*  stack *  pointer to the new stack.
*
* DESCRIPTION:
*  creates an empty stack.
*
* UPDATES: 
*
*********************************************************/
stack *stknew(nDataSize)
int nDataSize;
{
   stack *lpStack;
   
   lpStack = (stack *)malloc(sizeof(stack));
   if (lpStack != NULL)
   {
      lpStack->nDataSize = nDataSize;
      lpStack->lpTop = NULL;
   }

   return(lpStack);
}

/*********************************************************
* FUNCTION:
*  stkfree.
*
* PROTOTYPE:
*  int stkfree(stack *);
*
* PARAMETERS:
*  stack *lpStackName   stack header.
*
* RETURN:
*  STKNOERROR.
*
* DESCRIPTION:
*  free a stack, removing all of its elements.
*
* NOTES:
*
* UPDATES: 
*
*********************************************************/
int stkfree(lpStackName)
stack *lpStackName;
{
   stackelement *lpCurrent;

   if (lpStackName)
   {
      while(lpStackName->lpTop != NULL)
      {
         lpCurrent = (stackelement *)lpStackName->lpTop;
         lpStackName->lpTop = (stackelement *)lpStackName->lpTop->lpNext;
         free(lpCurrent->lpData);
         free(lpCurrent);
      }
      free(lpStackName);
   }
   return(STKNOERROR);
}

/*********************************************************
* FUNCTION:
*  stkpush.
*
* PROTOTYPE:
*  int stkpush(stack *, void *);
*
* PARAMETERS:
*  stack    *lpStackName  Stack definition.
*  void     *lpValue      Value to be inserted.
*
* RETURN:
*  int   STKNOERROR.
*        STKGENERROR error inserting, out of heap.
*
* DESCRIPTION:
*  add a new element to the to of the stack.
*
* NOTES:
*
* UPDATES:  
*
*********************************************************/
int stkpush(lpStackName, lpValue)
stack *lpStackName;
void  *lpValue;
{
   stackelement *lpElement;
   void        *lpData;
   int         nError = STKNOERROR; 

   lpElement = (stackelement *)malloc(sizeof(stackelement));
   lpData = malloc(lpStackName->nDataSize);
   if ((lpElement != NULL) && (lpData != NULL))
   {
      memcpy(lpData,lpValue,lpStackName->nDataSize);
      lpElement->lpData = lpData;

      lpElement->lpNext = (struct stackelement *)lpStackName->lpTop;
      lpStackName->lpTop = lpElement;
   }
   else
      nError = STKGENERROR;

   return(nError);
}


/*********************************************************
* FUNCTION:
*  stkpop.
*
* PROTOTYPE:
*  int stkpop(stack *, stackelement *);
*
* PARAMETERS:
*  stack *lpStackName   Stack definition.
*  void  *lpData        Data returned.
*
* RETURN:
*  int  STKNOERROR.
*       STKEMPTY    stack is empty.
*
* DESCRIPTION:
*  remove the data from the top of the stack.
*
* NOTES:
*
* UPDATES:  
*
*********************************************************/
int stkpop(lpStackName, lpData)
stack  *lpStackName;
void   *lpData;
{
   stackelement *lpCurrent;
   int          nError = STKEMPTY;

   if (lpStackName->lpTop)
   {
      /* get top element anrd remove it from stack */
      lpCurrent = lpStackName->lpTop;
      lpStackName->lpTop = (stackelement *)lpCurrent->lpNext;
      memcpy(lpData,lpCurrent->lpData,lpStackName->nDataSize);
      free(lpCurrent->lpData);
      free(lpCurrent);
      nError = STKNOERROR;
   }
   return(nError);
}

/*********************************************************
* FUNCTION:
*  stknumber.
*
* PROTOTYPE:
*  int stknumber(stack *);
*
* PARAMETERS:
*  stack    *lpStackName   Stack definition.
*
* RETURN:
*  int number of elements in the stack.
*
* DESCRIPTION:
*  return the number of elements in the stack.
*
* NOTES:
*
* UPDATES:
*
*********************************************************/
int stknumber(lpStackName)
stack *lpStackName;
{
   int nNum = 0;
   stackelement *lpStackElement;

   lpStackElement = lpStackName->lpTop;

   while(lpStackElement != NULL)
   {
      nNum ++;
      lpStackElement = (stackelement *)lpStackElement->lpNext;
   }
   
   return(nNum);
}


/*********************** DEBUG OPTION *******************/

/*********************************************************
* FUNCTION:
*  stkdump.
*
* PROTOTYPE:
*  int stkdump(stack *);
*
* PARAMETERS:
*  stack *lpStackName   stack header.
*
* RETURN:
*  STKNOERROR.
*
* DESCRIPTION:
*  dump a stack.
*
* NOTES:
*
* UPDATES: 
*
*********************************************************/
int stkdump(lpStackName)
stack *lpStackName;
{
   stackelement *lpElement;

   if (lpStackName)
   {
      fprintf(stdout,"Top %x \n",lpStackName->lpTop);
      lpElement = lpStackName->lpTop;
      while(lpElement != NULL)
      {
         fprintf(stdout,"Next %x Data %x\n",lpElement->lpNext,
                                            lpElement->lpData);
         lpElement = (stackelement *)lpElement->lpNext;
      }
   }

   return (STKNOERROR);
}

